﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.UI;

namespace _978950_RadGridViewPrintingHeaders
{
    public class CustomRadGridViewDragDropService : RadGridViewDragDropService
    {
        public CustomRadGridViewDragDropService(RadGridViewElement gridViewElement)
            : base(gridViewElement) { }

        public override string Name
        {
            get { return typeof(RadGridViewDragDropService).Name; }
        }

        protected override void HandleMouseMove(Point mousePosition)
        {
            base.HandleMouseMove(mousePosition);
            ISupportDrop dropTarget = this.DropTarget;
            this.AutoScroll();

            RadElement targetElement = dropTarget as RadElement;
            RadPosition newPosition = RadPosition.None;
            if (targetElement != null)
            {
                newPosition = this.GetDropPosition(targetElement.PointFromScreen(mousePosition), targetElement.ControlBoundingRectangle.Size);
            }

            RadPosition lastDropPosition = RadPosition.None;
            FieldInfo fiLastPosition = this.GetType().BaseType.GetField("lastDropPosition", BindingFlags.Instance | BindingFlags.NonPublic);
            if (fiLastPosition != null)
            {
                lastDropPosition = (RadPosition)fiLastPosition.GetValue(this);
            }

            if (dropTarget != this.DropTarget || dropTarget == null || newPosition != lastDropPosition)
            {
                this.SetDragDropBehavior();
            }

            lastDropPosition = newPosition;
            FieldInfo fiDragHintWindow = this.GetType().BaseType.GetField("dragHintWindow", BindingFlags.Instance | BindingFlags.NonPublic);
            RadLayeredWindow dragHintWindow = fiDragHintWindow.GetValue((RadDragDropService)this) as RadLayeredWindow;
            if (dragHintWindow == null)
            {
                return;
            }

            this.UpdateDragHintLocation(mousePosition);
        }

        private void AutoScroll()
        {
            Point location = this.GridViewElement.ElementTree.Control.PointToClient(Control.MousePosition);
            GridTableElement tableElement = this.GetTableElementAtPoint(location);

            ISupportDrag supportDrag = this.Context as ISupportDrag;
            object dataContext = supportDrag.GetDataContext();

            if (this.AllowAutoScrollRowsWhileDragging && dataContext is GridViewRowInfo)
            {
                this.ScrollRows(tableElement, location);
            }
            else if (this.AllowAutoScrollColumnsWhileDragging && dataContext is GridViewColumn)
            {
                this.ScrollColumns(tableElement, location);
            }
            else if (dataContext is GroupFieldDragDropContext)
            {
                this.ScrollColumns(tableElement, location);
            }
        }

        private void ScrollRows(GridTableElement tableElement, Point location)
        {
            ScrollableRowsContainerElement scrollableRows = tableElement.ViewElement.ScrollableRows;
            RadScrollBarElement vScrollbar = this.GetVeritcalScrollbar(tableElement);
            Rectangle containerBounds = scrollableRows.ControlBoundingRectangle;

            if (containerBounds.Contains(location) ||
                location.X < containerBounds.X ||
                location.X > containerBounds.Right)
            {
                return;
            }

            int delta = 0;

            if (location.Y > containerBounds.Bottom)
            {
                delta = location.Y - containerBounds.Bottom;
            }
            else if (location.Y < containerBounds.Y)
            {
                delta = location.Y - containerBounds.Y;
            }

            if (delta != 0 && vScrollbar.Visibility == ElementVisibility.Visible)
            {
                vScrollbar.Value = ClampValue(vScrollbar.Value + delta,
                                              vScrollbar.Minimum,
                                              vScrollbar.Maximum - vScrollbar.LargeChange + 1);
            }
        }

        private void ScrollColumns(GridTableElement tableElement, Point location)
        {
            GridTableHeaderRowElement headerRowElement = tableElement.GetRowElement(tableElement.ViewInfo.TableHeaderRow)
                                                            as GridTableHeaderRowElement;

            if (headerRowElement == null)
            {
                return;
            }

            Rectangle containerBounds = headerRowElement.ControlBoundingRectangle;

            containerBounds = headerRowElement.ScrollableColumns.ControlBoundingRectangle;

            int delta = 0;

            if (location.X > containerBounds.Right)
            {
                delta = location.X - containerBounds.Right;
            }
            else if (location.X < containerBounds.X)
            {
                delta = location.X - containerBounds.X;
            }

            RadScrollBarElement hScrollbar = tableElement.HScrollBar;

            if (tableElement.RightToLeft)
            {
                delta *= -2;
            }

            if (delta != 0 && hScrollbar.Visibility == ElementVisibility.Visible)
            {
                hScrollbar.Value = ClampValue(hScrollbar.Value + delta,
                                              hScrollbar.Minimum,
                                              hScrollbar.Maximum - hScrollbar.LargeChange + 1);
            }
        }

        private int ClampValue(int value, int minimum, int maximum)
        {
            if (value < minimum)
            {
                return minimum;
            }
            if (maximum > 0 && value > maximum)
            {
                return maximum;
            }
            return value;
        }

        private RadPosition GetDropPosition(Point dropLocation, Size size)
        {
            RadPosition result = RadPosition.None;

            int halfWidth = size.Width / 2;
            int thirdWidth = size.Width / 3;
            int twoThirdsWidth = 2 * size.Width / 3;

            if (dropLocation.X <= halfWidth)
                result |= RadPosition.Left;
            else
                result |= RadPosition.Right;

            if (thirdWidth < dropLocation.X && dropLocation.X < twoThirdsWidth)
            {
                int halfHeight = size.Height / 2;

                if (dropLocation.Y <= halfHeight)
                    result |= RadPosition.Top;
                else
                    result |= RadPosition.Bottom;
            }

            return result;
        }

        private RadScrollBarElement GetVeritcalScrollbar(GridTableElement tableElement)
        {
            if (GridViewElement.UseScrollbarsInHierarchy)
            {
                return tableElement.VScrollBar;
            }
            return GridViewElement.TableElement.VScrollBar;
        }
    }
}
